@@ -19,6 +19,11 @@ def package_installed_p_yum(package):
     res = scb("yum list installed '{package}' || true")
     return "Error: No matching Packages to list" not in res
 
+def package_installed_p_rpm(package):
+    res = scb("rpm -q '{package}' || true")
+    return "is not installed" not in res
+
+
 def install_package(package, url=None):
     if isinstance(package, list):
         res = False
@@ -38,6 +43,16 @@ def install_package(package, url=None):
                 fname = wget(url)
                 bash(lf("{su}dpkg -i {fname}"))
             return True
+    elif el.file_exists_p("/usr/bin/rpm") and el.file_exists_p("/usr/bin/zypper"):
+        if package_installed_p_rpm(package):
+            print(lf("{package}: OK"))
+            return False
+        else:
+            if url is None:
+                bash(lf("{su}zypper install -y {package}"))
+            else:
+                raise RuntimeError("Not yet implemented")
+            return True
     else:
         if package_installed_p_yum(package):
             print(lf("{package}: OK"))
@@ -54,13 +69,31 @@ def apt_key_add(email, url):
     if grep("apt-key list", email):
         print(email + " key: OK")
     else:
-        bash(lf("wget -qO - {url} | apt-key add -"))
+        install_package("gnupg2")
+        bash(lf("wget -qO - {url} | sudo apt-key add -"))
+
+def apt_add_repository(url, categories, gpg_key=None):
+    if gpg_key is None:
+        distro = scb("lsb_release -cs")
+        line = f"deb {url} {distro} {categories}"
+        cmd = f"sudo apt-add-repository '{line}'"
+    else:
+        gpg_key_url = repo_url + gpg_key
+        gpg_key_file = "/usr/share/keyrings/" + gpg_key
+        make(gpg_key_file, f"wget -O $1 {gpg_key_url}")
+        line = f"deb [signed-by={gpg_key_file}] {repo_url} {categories}"
+        cmd = f"echo 'line' | sudo tee {fname}"
+    fname = "/etc/apt/sources.list"
+    if line in el.slurp_lines(fname):
+        print(fname + ": OK")
+    else:
+        bash(cmd)
 
 def debconf_select(package, var, v_type, v_val):
     bash(lf("echo '{package} {var} {v_type} {v_val}' | debconf-set-selections"))
 
 def wget(url, download_dir="/tmp/"):
-    if download_dir[:-1] == "/":
+    if download_dir[-1] == "/":
         fname = url.split("/")[-1]
         full_name = el.expand_file_name(fname, download_dir)
     else:
@@ -69,7 +102,7 @@ def wget(url, download_dir="/tmp/"):
         print(lf("{full_name}: OK"))
     else:
         bash(sudo(lf("wget '{url}' -O {full_name}"), full_name))
-        return full_name
+    return full_name
 
 def systemctl_start(service):
     if not el.scb("systemctl is-active {service} || true") == "active\n":
@@ -107,7 +140,10 @@ def git_clone(url, target_dir, commit=None):
             sc("cd {gdir} && git reset --hard {commit}")
 
 def symlink_p(fname):
-    return " -> " in el.sc("stat {fname}")
+    (host, name) = parse_fname(fname)
+    with hostname(host):
+        txt = el.sc("stat {name}")
+    return " -> " in txt
 
 def sudo(cmd, fname=None):
     if fname:
@@ -122,25 +158,24 @@ def sudo(cmd, fname=None):
     return su + cmd
 
 def ln(fr, to):
-    fr = el.expand_file_name(fr)
-    if not el.file_exists_p(fr):
-        raise RuntimeError("File doesn't exist", fr)
-    to = el.expand_file_name(to)
-    if el.file_directory_p(to) and not el.file_directory_p(fr):
-        to_full = el.expand_file_name(el.file_name_nondirectory(fr), to)
-    else:
-        to_full = to
+    fr_full = el.expand_file_name(fr)
+    if not el.file_exists_p(fr_full):
+        raise RuntimeError("File doesn't exist", fr_full)
+    to_full = el.expand_file_name(to)
     if el.file_exists_p(to_full):
         if symlink_p(to_full):
             print(lf("{to_full}: OK"))
         else:
-            if file_equal(fr, to_full):
+            if file_equal(fr_full, to_full):
                 print(lf("{to_full} exists, contents equal"))
             else:
                 print(lf("{to_full} exists, contents NOT equal"))
     else:
-        fr_abbr = os.path.relpath(fr, os.path.dirname(to))
-        cmd = sudo(lf("ln -s {fr_abbr} {to_full}"), to_full)
+        if el.HOST:
+            cmd = lf("ln -s {fr} {to}")
+        else:
+            fr_abbr = os.path.relpath(fr_full, os.path.dirname(to))
+            cmd = sudo(lf("ln -s {fr_abbr} {to_full}"), to_full)
         bash(cmd)
 
 def file_equal(f1, f2):
@@ -148,10 +183,12 @@ def file_equal(f1, f2):
         return el.sc("md5sum " + shlex.quote(f)).split(" ")[0]
     return md5sum(f1) == md5sum(f2)
 
-def cp_host(fr, to):
+def cp_host(fr, to=None):
     # if el.file_exists_p(to) and file_equal(fr, to):
     #     print(lf("{to}: OK"))
     #     return False
+    if to is None:
+        to = el.file_name_nondirectory(fr).replace("_", "/")
     host = el.HOST
     with el.hostname(None):
         fr = el.expand_file_name(fr)
@@ -329,7 +366,7 @@ def patch(fname, patches):
         elif (re.search("^" + re.escape(chunk_before), txt, re.MULTILINE) and
               not re.search("^" + re.escape(chunk_after), txt, re.MULTILINE)):
             no_change = False
-            txt = re.sub("^" + re.escape(chunk_before), chunk_after, txt, flags=re.MULTILINE)
+            txt = txt.replace(chunk_before, chunk_after)
         else:
             # either already patched or fail
             if not chunk_after in txt:
@@ -348,3 +385,26 @@ def patch(fname, patches):
         with el.hostname(None):
             bash(cmd, desc=(host, "edit " + name))
         return True
+
+def install_root_cert(certfile, certname=None):
+    """Install root certificate to certificate trust store of applications using NSS"""
+    install_package("libnss3-tools")
+    name_crt = el.file_name_nondirectory(certfile) if certname is None else certname
+    # Firefox and Chromium
+    if not el.file_exists_p("/tmp/cert9.db.done"):
+        dbs = el.sc_l("find ~/ -name cert9.db 2>/dev/null || true")
+        for db in dbs:
+            certdir = el.file_name_directory(db)
+            el.scb("certutil -A -n '{name_crt}' -t 'TCu,Cu,Tu' -i {certfile} -d sql:{certdir}")
+        el.barf("/tmp/cert9.db.done", "")
+    # curl
+    cp_cmd = cp_host if el.HOST else cp
+    fname_crt = lf("/usr/local/share/ca-certificates/{name_crt}")
+    cp_cmd(certfile, fname_crt)
+    chmod(fname_crt, "644")
+    name_pem = el.file_name_sans_extension(name_crt) + ".pem"
+    fname_pem = lf("/etc/ssl/certs/{name_pem}")
+    make(fname_pem, sudo("update-ca-certificates"))
+    # wget
+    install_package("ca-certificates")
+    patch("/etc/wgetrc", ["+ca_directory=/etc/ssl/certs"])
